
module circle_check(
input i_clk,
input i_reset,
input i_chstart,
input [10:0] i_loc_x,//VGA_X
input [10:0] i_loc_y,//VGA_Y
input i_disp_bit,
output reg o_done,
output reg [21:0] r_diff,
output reg [6:0] o_damage,//damage from 1 to 128
output reg [21:0] o_bottom,//lower bits -- y, higher bits -- x
output reg [21:0] o_left,
output reg [21:0] o_right
);
/******************************************************************/
/* circle check						
/******************************************************************/ 
reg [9:0] point_cnt;// together with line 642 to check start point
wire done;
reg start,reset_to, check_start; //start and reset signals
reg [10:0] vga_x, vga_y;
reg [21:0] up_most_point, bottom_point, left_most_point, right_most_point, up_most_point1, bottom_point1, left_most_point1, right_most_point1;
wire [21:0] w_up_most_point, w_bottom_point, w_left_most_point, w_right_most_point, w_r_diff;
   
boundary_square_detect bsd(
.i_clk(i_clk),
.i_reset(reset_to),
.i_start(start),
.i_vga_x(vga_x),
.i_vga_y(vga_y),
.i_up_most_point(up_most_point),//lower bits -- y, higher bits -- x
.i_bottom_point(bottom_point),
.i_left_most_point(left_most_point),
.i_right_most_point(right_most_point),
.o_done(done),
.test(w_r_diff),
.o_up_most_point(w_up_most_point),//lower bits -- y, higher bits -- x
.o_bottom_point(w_bottom_point),
.o_left_most_point(w_left_most_point),
.o_right_most_point(w_right_most_point)
);

reg rd_start;
reg [21:0] circle_cen, longest_r, shortest_r;
wire [21:0] w_longest_r, w_shortest_r;
wire rd_done;
radius_diff rd(
.i_clk(i_clk),
.i_reset(reset_to),
.i_start(rd_start),
.i_vga_x(vga_x),
.i_vga_y(vga_y),
.i_circle_cen(circle_cen),//lower bits -- y, higher bits -- x
.i_longest_r(longest_r),
.i_shortest_r(shortest_r),
.o_done(rd_done),
.o_longest_r(w_longest_r),
.o_shortest_r(w_shortest_r)
);

reg [21:0] tmp1, tmp2, cir_cen;//, r_diff;
reg cir_center_found;
reg [10:0] start_x, start_y;//together with line 404 - 409 to record the start point
reg [3:0] state_c;
reg [2:0] counter;
parameter check_init = 4'd0, find_boundary_1 = 4'd1, find_boundary_2 = 4'd2, find_cir_center_1 = 4'd3, find_cir_center_2 = 4'd4, 
		find_radius_diff_1 = 4'd5, find_radius_diff_2 = 4'd6, cal_damage = 4'd7, init = 4'd8;
  
always@(posedge i_clk) begin
	if (i_reset) begin
		reset_to <= 1'b1;// ? 1 or 0??
		start <= 1'b0;
		up_most_point <= 22'b1111111111111111111111;
		bottom_point <= 22'd0;
		left_most_point <= 22'b1111111111111111111111;
		right_most_point <= 22'd0;
		cir_center_found <= 1'b0;
		state_c <= 4'd8;
		o_done <= 1'b0;
		o_damage <= 8'd0;
		o_bottom <= 22'd0;
		o_left <= 22'd0;
		o_right <= 22'd0;
		vga_x <= 11'd0;
		vga_y <= 11'd0;
		start_x <= 11'd1024;
		start_y <= 11'd1024;
		counter <= 3'd0;
		r_diff <= 22'd0;
	end else if (i_chstart) begin
		case (state_c)
			init: begin
				cir_center_found <= 1'b0;
				reset_to <= 1'b0;
				start <= 1'b0;
				o_done <= 1'b0;
				point_cnt <= 10'd0;
				counter <= 3'd0;
				state_c <= check_init;
			end
			check_init: begin
				reset_to <= 1'b0;
				start <= 1'b0;
				o_done <= 1'b0;
				// if the point is on the circle track and the circle centre not set
				if(i_disp_bit && !cir_center_found) begin
					// r_diff <= {i_loc_x, i_loc_y};
					if ((start_x == i_loc_x) && (start_y == i_loc_y)) begin// point repeats -- stop recording points
						point_cnt <= 10'd0;
						
						//o_done <= 1'b1;
						state_c <= find_cir_center_1;
					end else begin// point not repeats -- keep finding boundary
						vga_x <= i_loc_x;
						vga_y <= i_loc_y;
						point_cnt <= point_cnt + 10'd1;
						state_c <= find_boundary_1;
						if (point_cnt == 10'd0) begin// when point_cnt is 1, it is the start point, else the circle already starts
							start_x <= i_loc_x;
							start_y <= i_loc_y;
							counter <= 3'd0;
						end
					end
				// if the point is on the circle track and the circle centre set
				end else if (i_disp_bit && cir_center_found) begin
					if ((start_x == i_loc_x) && (start_y == i_loc_y)) begin// point repeats -- stop recording points
						point_cnt <= 10'd0;
						state_c <= find_radius_diff_2;// find the radius difference for ball character use
						
					end else begin// point not repeats -- keep comparing radius
						vga_x <= i_loc_x;
						vga_y <= i_loc_y;
						point_cnt <= point_cnt + 10'd1;
						state_c <= find_radius_diff_1;
						if (point_cnt == 10'd1) begin// when point_cnt is 1, it is the start point, else the circle already starts
							start_x <= i_loc_x;
							start_y <= i_loc_y;
						end
					end
				end
			end
			find_boundary_1: begin
				reset_to <= 1'b0;
				start <= 1'b1;
				
				if (done) begin
					counter <= counter + 3'd1;
					up_most_point1 <= w_up_most_point;
					bottom_point1 <= w_bottom_point;
					left_most_point1 <= w_left_most_point;
					right_most_point1 <= w_right_most_point;
					
					
					state_c <= find_boundary_2;
					
				end
			end
			find_boundary_2: begin
				//r_diff <= w_r_diff;
				up_most_point <= up_most_point1;//w_up_most_point;
				bottom_point <= bottom_point1;//w_bottom_point;
				left_most_point <= left_most_point1;//w_left_most_point;
				right_most_point <= right_most_point1;//w_right_most_point;
				
				//if (counter == 3'd1) begin
					o_left <= left_most_point1;//w_left_most_point;
						
				//end
				//else if (counter == 3'd2)
					o_bottom <= bottom_point1;//w_bottom_point;
				//else if (counter == 3'd3)
					o_right <= right_most_point1;//w_right_most_point;
				// else if (counter == 3'd4) begin
					// r_diff <= w_right_most_point;	
				// end
				// o_bottom <= w_bottom_point;
				// o_left <= w_left_most_point;
				// o_right <= w_right_most_point;
				// r_diff <= w_right_most_point;
				state_c <= check_init;
			end
			find_cir_center_1: begin
				
				tmp1[10:0] <= ((bottom_point[10:0] - up_most_point[10:0]) >> 1) + up_most_point[10:0];  //y coordinate
				tmp1[21:11] <= (up_most_point[21:11] < bottom_point[21:11])?(((bottom_point[21:11] - up_most_point[21:11]) >> 1) + up_most_point[21:11])
							:(((up_most_point[21:11] - bottom_point[21:11]) >> 1) + bottom_point[21:11]); //x coordinate
							
				tmp2[21:11] <= ((right_most_point[21:11] - left_most_point[21:11]) >> 1) + left_most_point[21:11]; //x coordinate
				tmp2[10:0] <= (left_most_point[10:0] < right_most_point[10:0])?(((right_most_point[10:0] - left_most_point[10:0]) >> 1) + left_most_point[10:0])
							:(((left_most_point[10:0] - right_most_point[10:0]) >> 1) + right_most_point[10:0]); //y coordinate
							
				state_c <= find_cir_center_2;
			end
			find_cir_center_2: begin
				cir_cen[10:0] <= ((tmp1[10:0] + tmp2[10:0]) >> 1);
				cir_cen[21:11] <= ((tmp1[21:11] + tmp2[21:11]) >> 1);
				cir_center_found <= 1'b1;
				start_x <= 11'd1024;//reset out of 640x480 for center use
				start_y <= 11'd1024;//reset out of 640x480 for center use
				state_c <= check_init;//?
				
			end
			find_radius_diff_1: begin
				reset_to <= 1'b0;
				rd_start <= 1'b1;
				circle_cen <= cir_cen;
				
				if (rd_done) begin
					longest_r <= w_longest_r;
					shortest_r <= w_shortest_r;
					state_c <= check_init;
				end
			end
			find_radius_diff_2: begin
				reset_to <= 1'b0;
				rd_start <= 1'b0;
				r_diff <= longest_r - shortest_r;
				
				state_c <= cal_damage;
				// ledr <= r_diff[17:0];
			end
			cal_damage: begin
				o_damage <= 7'd128 - r_diff[18:12];
				o_done <= 1'b1;/* signals others that circle check is finished*/
			end
		endcase
	end
	// if(mem_bit) begin
		// ledg[2] <= ledg[2]^1'd1;
	// end
	// ledg[7:3] <= state_c;
end 
endmodule
