module vga_line(
	input clk,
	input reset,
	
	input signed [10:0] ix1,
	input signed [10:0] iy1,
	input signed [10:0] ix2,
	input signed [10:0] iy2,
	
	input istart_draw,

	output reg [10:0] ox_out,
	output reg [10:0] oy_out,
	
	output reg oidle
);

parameter idle=4'd0, init=4'd1, init1=4'd2, init2=4'd3, init3=4'd4, loop=4'd5, loop1=4'd6, loop2=4'd7;
parameter endloop=4'd8;
reg [3:0] state;


reg signed [10:0] j,dx,dy,x,y,e;
reg signed [1:0] s1,s2,xchange;

always@(posedge clk) begin
	if(reset) begin
		state <=  idle;
		ox_out <= 1'd0;
		oy_out <= 1'd0;
		oidle <= 1'd0;
		
		x <= 0;
		y <= 0;
	end else begin
		case(state)
			idle:
			begin
				j <= 0;
				dx <= 0;
				dy <= 0;

				e <= 0;
				s1 <= 0;
				s2 <= 0;
				xchange <= 0;
			
			
				if(istart_draw) begin
					state <= init1;
					oidle <= 1'd0;
				end else begin
					oidle <= 1'd1;
				end
			end
			
			init1:
			begin
				x <= ix1;
				y <= iy1;
				
				if(ix2<ix1) begin
					dx <= ix1-ix2;
					s1 <= -1;
				end else if(ix2 == ix1) begin
					dx <= 0;
					s1 <= 0;
				end else begin
					dx <= ix2-ix1;
					s1 <= 1;
				end
				
				if(iy2<iy1) begin
					dy <= iy1-iy2;
					s2 <= -1;
				end else if(iy2 == iy1) begin
					dy <= 0;
					s2 <= 0;
				end else begin
					dy <= iy2-iy1;
					s2 <= 1;
				end
				
				xchange <= 0;
				
				state <= init2;
			end
			
			init2:
			begin
				ox_out <= x;
				oy_out <= y;
			
				if(dy>dx) begin
					dx <= dy;
					dy <= dx;
					xchange <= 1;
				end
				
				state <= init3;
			end
			
			init3:
			begin
				e <= (dy<<1)-dx;
				
				state <= loop;
			end
			
			loop:
			begin
				if(e>=0) begin
					if(xchange==1) begin
						x <= x+s1;
					end else begin
						y <= y+s2;
					end
					e <= e-(dx<<1);
				end
				
				j <= j+1;
				state <= loop1;
			end
			
			loop1:
			begin
				if(xchange == 1) begin
					y <= y+s2;
				end else begin
					x <= x+s1;
				end
				
				e <= e + (dy<<1);
				state <= loop2;
			end
			
			loop2:
			begin
				ox_out <= x;
				oy_out <= y;
				if(j<=dx) begin
					state <= loop;
				end else begin
					state <= endloop;
					oidle <= 1;
				end
			end
			
			endloop:
			begin
				state <= idle;
			end
		
		endcase
	end

end

endmodule